/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2012-12-04 10:58:10 +0900 #$
** $Revision: 3024 $
** $Author: kato@FITECHLABS.CO.JP $
**/

#include "common.h"
#include "comms.h"
#include "log.h"
#include "zbxjson.h"
#include "threads.h"

#include "jacommon.h"
#include "jastr.h"
#include "jarun.h"
#include "jagetagent.h"
#include "jaruniconjob.h"
#include "jalog.h"

extern char   serverid[JA_SERVERID_LEN];

int set_status_runerror(zbx_uint64_t inner_job_id, zbx_uint64_t inner_jobnet_id)
{
	const char *__function_name = "set_status_runerror";
	int ret = SUCCEED;
	int	result;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s()",__function_name);

	result = DBexecute("update ja_run_job_table set status = %d, start_time = '%s', end_time = '%s'"
			" where inner_job_id="ZBX_FS_UI64";\n"
			, JA_JOB_STATUS_RUNERR, ja_timestamp2str(time(NULL)), ja_timestamp2str(time(NULL)), inner_job_id);

	if ( ZBX_DB_OK > result) {
		ja_log("JARUNICONJOB200001", inner_jobnet_id, NULL, inner_job_id);
		ret = FAIL;
		return ret;
	}

	ja_joblog(JC_JOB_ERR_END, inner_jobnet_id, inner_job_id);
	ja_log("JARUNICONJOB200023", inner_jobnet_id, NULL, inner_job_id);

	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
	return ret;
}

int set_status_abort(zbx_uint64_t inner_job_id, zbx_uint64_t inner_jobnet_id)
{
	const char *__function_name = "set_status_abort";
	int ret = SUCCEED;
	int	result;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s()",__function_name);

	result = DBexecute("update ja_run_job_table set status = %d, start_time = '%s', end_time = '%s'"
			" where inner_job_id="ZBX_FS_UI64";\n"
			, JA_JOB_STATUS_ABORT, ja_timestamp2str(time(NULL)), "0", inner_job_id);

	if ( ZBX_DB_OK > result) {
		ja_log("JARUNICONJOB200001", inner_jobnet_id, NULL, inner_job_id);
		ret = FAIL;
		return ret;
	}

	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
	return ret;
}

int update_job_status(char *buf, zbx_uint64_t inner_job_id, zbx_uint64_t inner_jobnet_id, const char *host, int flag)
{
	const char *__function_name = "update_jobnet_status";
	struct 		zbx_json_parse jp, jp_row;
	char		value[MAX_STRING_LEN];
	const char	*p;
	char		*message;
	int		version;
	int		result;
	int ret = SUCCEED;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s() host: %s",__function_name, host);

	if ( SUCCEED == zbx_json_open(buf, &jp))
	{
		if (SUCCEED == zbx_json_value_by_name(&jp, JA_PROTO_TAG_KIND, value, sizeof(value)))
		{
			if (0 != strcmp(value, JA_PROTO_VALUE_JOBRUN_RES)) {
				ja_log("JARUNICONJOB200002", inner_jobnet_id, NULL, inner_job_id);
				if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
					ret = FAIL;
				}
				return ret;
			}
		} else {
			ja_log("JARUNICONJOB200003", inner_jobnet_id, NULL, inner_job_id);
			if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
				ret = FAIL;
			}
			return ret;
		}

		if (SUCCEED == zbx_json_value_by_name(&jp, JA_PROTO_TAG_VERSION, value, sizeof(value)))
		{
			version = atoi(value);
			if ( version != JA_PROTO_VALUE_VERSION_1) {
				ja_log("JARUNICONJOB200004", inner_jobnet_id, NULL, JA_PROTO_VALUE_VERSION_1, inner_job_id);
				if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
					ret = FAIL;
				}
				return ret;
			}
		} else {
			ja_log("JARUNICONJOB200005", inner_jobnet_id, NULL, inner_job_id);
			if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
				ret = FAIL;
			}
			return ret;
		}

		if (SUCCEED == zbx_json_value_by_name(&jp, JA_PROTO_TAG_SERVERID, value, sizeof(value)))
		{
			if (0 != strcmp(value, serverid)) {
				ja_log("JARUNICONJOB200006", inner_jobnet_id, NULL, serverid, inner_job_id, value);
				if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
					ret = FAIL;
				}
				return ret;
			}

		} else {
			ja_log("JARUNICONJOB200007", inner_jobnet_id, NULL, inner_job_id);
			if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
				ret = FAIL;
			}
			return ret;

		}

		if (NULL == (p = zbx_json_pair_by_name(&jp, JA_PROTO_TAG_DATA))) {
			ja_log("JARUNICONJOB200008", inner_jobnet_id, NULL, inner_job_id);
			if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
				ret = FAIL;
			}
			return ret;

		} else {
			if (FAIL == (ret = zbx_json_brackets_open(p, &jp_row))) {
				ja_log("JARUNICONJOB200009", inner_jobnet_id, NULL, inner_job_id);
				ret = FAIL;

			} else {
				if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_RESULT, value, sizeof(value))) {
					result = atoi(value);
					if ( result == JA_JOBRUN_RES_SUCCEED ) {
						if ( flag == JA_AGENT_METHOD_ABORT || flag == JA_AGENT_METHOD_KILL) {
							if (SUCCEED != set_status_abort(inner_job_id, inner_jobnet_id)) {
								ret = FAIL;
							}
						}
					} else if ( result == JA_JOBRUN_RES_VERSION_FAIL ) {

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_MYVERSION, value, sizeof(value))) {
							version = atoi(value);
							ja_log("JARUNICONJOB200010", inner_jobnet_id, NULL, inner_job_id, version);
							if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
								ret = FAIL;
							}
						} else {
							ja_log("JARUNICONJOB200011", inner_jobnet_id, NULL, inner_job_id);
							if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
								ret = FAIL;
							}
							return ret;
						}

					} else if ( result == JA_JOBRUN_RES_FAIL ) {
						
						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_MESSAGE, value, sizeof(value))) {
							message = strdup(value);
							ja_log("JARUNICONJOB200012", inner_jobnet_id, NULL, inner_job_id, message);
							if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
								ret = FAIL;
							}
						} else {
							ja_log("JARUNICONJOB200013", inner_jobnet_id, NULL, inner_job_id);
							if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
								ret = FAIL;
							}
							return ret;
						}

					} else if ( result == JA_JOBRUN_RES_ALREADY_RUN ) {						
						ja_log("JARUNICONJOB200026", inner_jobnet_id, NULL, inner_job_id);
					} else {
						ja_log("JARUNICONJOB200014", inner_jobnet_id, NULL, inner_job_id, result);
						if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
							ret = FAIL;
						}
						return ret;
					}

				} else {
					ja_log("JARUNICONJOB200015", inner_jobnet_id, NULL, inner_job_id);
					if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
						ret = FAIL;
					}
					return ret;
				}
			}
		}
	} else {
		ja_log("JARUNICONJOB200016", inner_jobnet_id, NULL, inner_job_id);
		if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
			ret = FAIL;
		}
	}
	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);

	return ret;
}

int jarun_icon_job(zbx_uint64_t inner_job_id, int flag)
{
   	const char *__function_name = "jarun_icon_job";
	zbx_sock_t s;
	pid_t	pid;
	char *buf;
	int ret = SUCCEED;
	int exit_code;
	zbx_uint64_t	inner_jobnet_id;
	zbx_uint64_t	inner_job_id_fs_link;
	struct zbx_json 	json;
	JA_AGENT_INFO 	agent_info;
	DB_RESULT	result;
	DB_RESULT	result_e;
	DB_ROW		row; 
	DB_ROW		row_e;
	int		command_type;
	const char	*command;
	char		*value_name;
	char		*value_name_e;
	char		*before_value;
	char		*value;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s()",__function_name);

	/*flag range check*/
	if ( flag < JA_AGENT_METHOD_NORMAL || flag > JA_AGENT_METHOD_KILL ) {
		ja_log("JARUNICONJOB200025", 0, NULL, inner_job_id, flag);
		if (SUCCEED != set_status_runerror(inner_job_id, 0)) {
			ret = FAIL;
		}
		return ret;
	}

	/*JSON Data*/
	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
	zbx_json_addstring(&json, JA_PROTO_TAG_KIND, JA_PROTO_VALUE_JOBRUN, ZBX_JSON_TYPE_STRING);
	zbx_json_adduint64(&json, JA_PROTO_TAG_VERSION, JA_PROTO_VALUE_VERSION_1); /*data version*/
	zbx_json_addstring(&json, JA_PROTO_TAG_SERVERID, serverid, ZBX_JSON_TYPE_STRING);
	zbx_json_addobject(&json, JA_PROTO_TAG_DATA);

	/*inner_job_id*/
	if ( flag == JA_AGENT_METHOD_ABORT ) {
		result = DBselect("select inner_job_id_fs_link from ja_run_job_table"
					" where inner_job_id = "ZBX_FS_UI64
					, inner_job_id);
		row = DBfetch(result);
		if ( row == NULL) {
			ja_log("JARUNICONJOB200024", 0, NULL, inner_job_id);
			DBfree_result(result);
			zbx_json_free(&json);
			if (SUCCEED != set_status_runerror(inner_job_id, 0)) {
				ret = FAIL;
			}
			return ret;
		}

		ZBX_STR2UINT64(inner_job_id_fs_link, row[0]); 
		zbx_json_adduint64(&json, JA_PROTO_TAG_JOBID, inner_job_id_fs_link);
		DBfree_result(result);

	} else {
		zbx_json_adduint64(&json, JA_PROTO_TAG_JOBID, inner_job_id);
	}

	/*KILL*/
	if ( flag == JA_AGENT_METHOD_KILL ) {
		zbx_json_adduint64(&json, JA_PROTO_TAG_METHOD, JA_AGENT_METHOD_KILL);
		zbx_json_close(&json);
		zbx_json_close(&json);
		goto senddata;
	}

	/*type*/
	result = DBselect("select inner_jobnet_id, command_type from ja_run_icon_job_table"
				" where inner_job_id = "ZBX_FS_UI64
				, inner_job_id);
	row = DBfetch(result);
	if ( row == NULL) {
		ja_log("JARUNICONJOB200017", 0, NULL, inner_job_id);
		DBfree_result(result);
		zbx_json_free(&json);
		if (SUCCEED != set_status_runerror(inner_job_id, 0)) {
			ret = FAIL;
		}
		return ret;
	}

	ZBX_STR2UINT64(inner_jobnet_id, row[0]); 
	command_type = atoi(row[1]);

	if ( command_type == JA_JOB_TYPE_SCRIPT) {
		zbx_json_addstring(&json, JA_PROTO_TAG_TYPE, JA_PROTO_VALUE_SCRIPT, ZBX_JSON_TYPE_STRING);
	} else if ( command_type == JA_JOB_TYPE_COMMAND ){
		zbx_json_addstring(&json, JA_PROTO_TAG_TYPE, JA_PROTO_VALUE_COMMAND, ZBX_JSON_TYPE_STRING);
	} else {
		ja_log("JARUNICONJOB200018", inner_jobnet_id, NULL, inner_job_id, command_type);
		DBfree_result(result);
		zbx_json_free(&json);
		if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
			ret = FAIL;
		}
		return ret;
	}
	
	DBfree_result(result);

	if ( flag == JA_AGENT_METHOD_NORMAL ) {
		zbx_json_adduint64(&json, JA_PROTO_TAG_METHOD, JA_AGENT_METHOD_NORMAL);
	} else if ( flag == JA_AGENT_METHOD_TEST ) {
		zbx_json_adduint64(&json, JA_PROTO_TAG_METHOD, JA_AGENT_METHOD_TEST);
	} else if ( flag == JA_AGENT_METHOD_ABORT ) {
		zbx_json_adduint64(&json, JA_PROTO_TAG_METHOD, JA_AGENT_METHOD_ABORT);
	} 

	/*env*/
	zbx_json_addobject(&json, JA_PROTO_TAG_ENV);

	result = DBselect("select value_name from ja_run_value_jobcon_table"
				" where inner_job_id = "ZBX_FS_UI64
				,inner_job_id);

	while (NULL != (row = DBfetch(result)))
	{
		value_name = NULL;
		before_value = NULL;

		value_name = DBdyn_escape_string(row[0]);

		result_e = DBselect("select before_value from ja_run_value_before_table"
				" where inner_job_id = "ZBX_FS_UI64
				" and value_name = '%s'"
				,inner_job_id,value_name);

		row_e = DBfetch(result_e);

		if ( row_e == NULL ) {
			zbx_json_addstring(&json, value_name,  "", ZBX_JSON_TYPE_STRING);
		} else {
 			before_value = row_e[0];
			zbx_json_addstring(&json, value_name,  before_value, ZBX_JSON_TYPE_STRING);
		}

		free(value_name);
		DBfree_result(result_e);
	}

	DBfree_result(result);

	result = DBselect("select value_name, value from ja_run_value_job_table"
				" where inner_job_id = "ZBX_FS_UI64
				,inner_job_id);

	while (NULL != (row = DBfetch(result)))
	{
		value_name = NULL;
		value = NULL;
		before_value = NULL;

		value_name = row[0];
		value = row[1];
		
		if ( value == NULL ) {
			zbx_json_addstring(&json, value_name,  "", ZBX_JSON_TYPE_STRING);
		} else {
			/*Check for variable*/
			if ((strncmp(value, "$", 1)) == 0 ){

				value++;	/*trim "$"*/
				value_name_e = DBdyn_escape_string(value);

				result_e = DBselect("select before_value from ja_run_value_before_table"
						" where inner_job_id = "ZBX_FS_UI64
						" and value_name = '%s'"
						,inner_job_id,value_name_e);

				row_e = DBfetch(result_e);
        	
				if ( row_e == NULL ) {
					zbx_json_addstring(&json, value_name,  "", ZBX_JSON_TYPE_STRING);
				} else {
 					before_value = row_e[0];
					zbx_json_addstring(&json, value_name,  before_value, ZBX_JSON_TYPE_STRING);
				}

				free(value_name_e);
				DBfree_result(result_e);
			} else {
				zbx_json_addstring(&json, value_name,  value, ZBX_JSON_TYPE_STRING);
			}
		}
	}

	DBfree_result(result);
	zbx_json_close(&json);

	/*command*/
	if ( flag == JA_AGENT_METHOD_NORMAL || flag == JA_AGENT_METHOD_TEST ) {
		result = DBselect("select command from ja_run_job_command_table"
				" where inner_job_id = "ZBX_FS_UI64
				" and command_cls = %d"
				,inner_job_id, command_type);
	} else if ( flag == JA_AGENT_METHOD_ABORT ) {
		result = DBselect("select command from ja_run_job_command_table"
				" where inner_job_id = "ZBX_FS_UI64
				" and command_cls =2"
				,inner_job_id);
	}

	row = DBfetch(result);

	if ( row != NULL ) {
		command = row[0];
		zbx_json_addstring(&json, JA_PROTO_TAG_SCRIPT,  command, ZBX_JSON_TYPE_STRING);
	}

	DBfree_result(result);
	zbx_json_close(&json);
	zbx_json_close(&json);

senddata:
	/*fork*/
	pid = zbx_fork();
	if ( pid == -1) {
		ja_log("JARUNICONJOB200027", inner_jobnet_id, NULL, inner_job_id);
		if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
			ret = FAIL;
		}
		return ret;
	} else if ( pid != 0) {
		waitpid(pid, &exit_code, 0);
		if ( exit_code != 0 ) {
			if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
				ret = FAIL;
			}
		}
		return ret;
	}

	pid = zbx_fork();
	if ( pid == -1) {
		ja_log("JARUNICONJOB200028", inner_jobnet_id, NULL, inner_job_id);
		exit(1);
	} else if ( pid != 0 ) {
		waitpid(pid, NULL, WNOHANG);
        	exit(0);
	}

	DBconnect(ZBX_DB_CONNECT_ONCE);

	/*Send JSON data to agent*/
	if (SUCCEED == get_agent_info(inner_job_id, inner_jobnet_id, &agent_info))
	{
		zabbix_log(LOG_LEVEL_DEBUG, "JSON before sending [%s]", json.buffer);

		if ( SUCCEED == zbx_tcp_connect(&s, CONFIG_SOURCE_IP, agent_info.host, agent_info.port, CONFIG_TIMEOUT))
		{
			if (SUCCEED == zbx_tcp_send_ext(&s, json.buffer, ZBX_TCP_PROTOCOL,CONFIG_TIMEOUT))
			{
				if (SUCCEED == SUCCEED_OR_FAIL(zbx_tcp_recv_ext(&s, &buf, 0 ,CONFIG_TIMEOUT))) 
				{
					zbx_rtrim(buf, " \r\n");
					zbx_ltrim(buf, " ");

					zabbix_log(LOG_LEVEL_DEBUG, "JSON back [%s]",buf);
					if (SUCCEED != update_job_status(buf, inner_job_id, inner_jobnet_id, agent_info.host, flag)) {
						ret = FAIL;
					}
				} else {
					ja_log("JARUNICONJOB200019", inner_jobnet_id, NULL, agent_info.host, zbx_tcp_strerror());
					if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
						ret = FAIL;
					}
				}

			} else {
				ja_log("JARUNICONJOB200020", inner_jobnet_id, NULL, agent_info.host, zbx_tcp_strerror());
				if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
					ret = FAIL;
				}
			}

			zbx_tcp_close(&s);

		} else {
			ja_log("JARUNICONJOB200021", inner_jobnet_id, NULL, zbx_tcp_strerror());
			if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
				ret = FAIL;
			}
		}

	} else {
		ja_log("JARUNICONJOB200022", inner_jobnet_id, NULL, inner_job_id);
		if (SUCCEED != set_status_runerror(inner_job_id, inner_jobnet_id)) {
			ret = FAIL;
		}
	}

	zbx_json_free(&json);

	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);

   	exit(0);
}
