/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2012-12-05 10:37:50 +0900 #$
** $Revision: 3049 $
** $Author: kato@FITECHLABS.CO.JP $
**/

#include "common.h"
#include "comms.h"
#include "log.h"
#include "md5.h"
#include "zbxserver.h"
#include "zbxself.h"

#include "jacommon.h"
#include "jatrapper.h"
#include "javalue.h"
#include "jagetagent.h"
#include "jaflow.h"
#include "jastr.h"
#include "jalog.h"

extern unsigned char process_type;
extern char   serverid[JA_SERVERID_LEN];

void init_agent_result(JOBARG_AGENT_RESULT *ar)
{
	ar->host = NULL;
	ar->jobid = 0;
	ar->result = 0;
	ar->message = NULL;
	ar->jobstdout = NULL;
	ar->jobstderr = NULL;
	ar->ret = NULL;
}

void init_exec_request(JOBARG_EXEC_REQUEST *er)
{
	er->host = NULL;
	er->username = NULL;
	er->password = NULL;
	er->jobnetid = NULL;
}

int host_auth(zbx_sock_t *sock, char  *host) 
{
	const char *__function_name = "host_auth";
	char		*host_value;
	char		*hostname = NULL;
	int 		ret = SUCCEED;
	int		hostid;
	int		useip;
	DB_RESULT	result;
	DB_ROW		row;
	
	zabbix_log(LOG_LEVEL_DEBUG, "In %s() host: [%s] ",__function_name, host);

	host_value = DBdyn_escape_string(host);
	result = DBselect("select hostid, useip from hosts where host='%s'"
				,host_value);

	row = DBfetch(result);

	if ( row == NULL ) {
		ja_log("JATRAPPER200001", 0, NULL, host_value);
		ret = FAIL;
		return ret;
	} 

	zbx_free(host_value);
	DBfree_result(result);

	hostid = atoi(row[0]);
	useip = atoi(row[1]);

	if ( useip == USE_IPADDRESS ) {
		/*IP address*/
		result = DBselect("select ip from hosts where hostid=%d"
					,hostid);
	} else if ( useip == USE_DNS ) {
		/*DNS*/
		result = DBselect("select dns from hosts where hostid=%d"
					,hostid);
	} else {
		ja_log("JATRAPPER200002", 0, NULL, useip);
		ret = FAIL;
		return ret;
	}
	
	row = DBfetch(result);

	if ( row == NULL) {
		ja_log("JATRAPPER200003", 0, NULL);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	DBfree_result(result);
	hostname = row[0];

	if ( FAIL == (zbx_tcp_check_security(sock, hostname, 0))){
		ja_log("JATRAPPER200004", 0, NULL, hostname, zbx_tcp_strerror());
		ret = FAIL;
		return ret;
	} 

	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);

	return ret;
}

int job_exec_auth(JOBARG_EXEC_REQUEST er)
{
	const char *__function_name = "job_exec_auth";
	int ret = SUCCEED;
	DB_RESULT	result;
	DB_ROW		row;
	md5_state_t	state;
	md5_byte_t	hash[MD5_DIGEST_SIZE];
	char		*jobnet_id;
	char		*user_name;
	char		*host_name;
	char		*passwd;
	char		*hash_text = NULL;
	int		nbytes;
	int		i;
	int		permission;
	size_t		sz;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s()",__function_name);

	/***User check****/
	jobnet_id = DBdyn_escape_string(er.jobnetid);
	user_name = DBdyn_escape_string(er.username);
	result = DBselect("select jobnet_id from ja_jobnet_control_table where jobnet_id = '%s' and valid_flag=1 and user_name in"
				" (select alias from users where userid in (select userid from users_groups where usrgrpid in"
				" (select g.usrgrpid from users_groups g, usrgrp u where g.usrgrpid=u.usrgrpid and u.users_status=0 and g.userid in"
				" (select userid from users where alias = '%s'))))", jobnet_id, user_name);

	row = DBfetch(result);
	if ( row == NULL) {
		ja_log("JATRAPPER200005", 0, NULL, jobnet_id, user_name);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	DBfree_result(result);

	/***Host check***/
	host_name = DBdyn_escape_string(er.host);
	result = DBselect("select permission from rights where groupid in"
				" (select usrgrpid from users_groups where userid in"
				" (select userid from users where alias = '%s'))"
				" and id in"
				" (select groupid from hosts_groups where hostid in"
				" (select hostid from hosts where host = '%s'))", user_name, host_name);

	int flag = 0;
	while (NULL != (row = DBfetch(result)))
	{
		flag = 1; /*row is not NULL*/
		permission = atoi(row[0]);
		if ( permission >= PERM_READ_ONLY ) {
			flag = 2;  /*permission OK*/
			break;
		}
	}

	if ( flag == 0 ) {
		/**row is NULL**/
		ja_log("JATRAPPER200006", 0, jobnet_id, user_name, host_name);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		zbx_free(host_name);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	} else if ( flag == 1 ) {
		/**permission NG**/
		ja_log("JATRAPPER200007", 0, jobnet_id);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		zbx_free(host_name);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	DBfree_result(result);

	/***Password check***/
	md5_init(&state);
	passwd = DBdyn_escape_string(er.password);
	nbytes = strlen(passwd);
	md5_append(&state, (const md5_byte_t *)passwd, nbytes);
	md5_finish(&state, hash);

	sz = MD5_DIGEST_SIZE * 2 + 1;
	hash_text = zbx_malloc(hash_text, sz);

	for (i = 0; i < MD5_DIGEST_SIZE; i++)
	{
		zbx_snprintf(&hash_text[i << 1], sz - (i << 1), "%02x", hash[i]);
	}

	result = DBselect("select userid, alias from users"
				" where alias = '%s'"
				" and passwd = '%s'", user_name, hash_text);

	row = DBfetch(result);
	if ( row == NULL) {
		ja_log("JATRAPPER200008", 0, jobnet_id);
		zbx_free(jobnet_id);
		zbx_free(user_name);
		zbx_free(host_name);
		zbx_free(passwd);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	DBfree_result(result);
	zbx_free(jobnet_id);
	zbx_free(user_name);
	zbx_free(host_name);
	zbx_free(passwd);

	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
	return ret;
}

int update_db_status(JOBARG_AGENT_RESULT ar)
{
	const char *__function_name = "update_db_status";
	int 		ret = SUCCEED;
	int		result;
	int		status;
	int		job_type;
	int		skipflag = 0;
	char		*range = NULL;
	zbx_uint64_t	inner_jobnet_id;
	DB_RESULT	result_j;
	DB_ROW		row;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s()",__function_name);

	if ( ar.method == JA_AGENT_METHOD_ABORT) {

		if ( ar.result == FAILURE) {
			ja_log("JATRAPPER200042", 0, NULL, ar.jobid, ar.message);

		}
		return ret;
	}


	result_j = DBselect("select inner_jobnet_id, job_type, status from ja_run_job_table"
				" where inner_job_id ="ZBX_FS_UI64
				, ar.jobid);

	row = DBfetch(result_j);
	if ( row == NULL) {
		ja_log("JATRAPPER200012", 0, NULL, ar.jobid);
		DBfree_result(result_j);
		return ret;
	}

	DBfree_result(result_j);
	ZBX_STR2UINT64(inner_jobnet_id, row[0]);
	job_type = atoi(row[1]);
	status = atoi(row[2]);

	switch (status)
	{

		case JA_JOB_STATUS_BEGIN:
			ja_log("JATRAPPER200010", inner_jobnet_id, NULL, ar.jobid);
			status = JA_JOBNET_STATUS_ENDERR;
			skipflag = 1;
			break;
		case JA_JOB_STATUS_READY:
			ja_log("JATRAPPER200010", inner_jobnet_id, NULL, ar.jobid);
			ret = FAIL;
			return ret;
		case JA_JOB_STATUS_ABORT:
			status = JA_JOBNET_STATUS_RUNERR;
			break;
		case JA_JOB_STATUS_RUN:
			if ( ar.result == FAILURE ) {
				status = JA_JOBNET_STATUS_RUNERR;
			} else {
				result_j = DBselect("select stop_code from ja_run_icon_job_table"
							" where inner_job_id = "ZBX_FS_UI64
							, ar.jobid);
				row = DBfetch(result_j);
				if ( row == NULL) {
					DBfree_result(result_j);
					status = JA_JOBNET_STATUS_END;
				} else {
					DBfree_result(result_j);
					range = row[0];

					if ( 0 == ja_number_match( ar.ret, range)) {
						status = JA_JOBNET_STATUS_END;
					} else {
						status = JA_JOBNET_STATUS_RUNERR;
					}
				}
			}
			break;
		default:
			return ret;

	}

	if ( status != JA_JOBNET_STATUS_END) {
		result = DBexecute("update ja_run_job_table set status = %d , end_time = '%s'" 
					" where inner_job_id="ZBX_FS_UI64";\n"
					, status, ja_timestamp2str(time(NULL)), ar.jobid);
		if ( ZBX_DB_OK > result) {
			ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid);
			ret = FAIL;
			return ret;
		}
	}

	if ( skipflag == 1) {
		ja_joblog(JC_JOB_ERR_END, inner_jobnet_id, ar.jobid);
		ja_log("JATRAPPER200043", inner_jobnet_id, NULL, ar.jobid);
		return ret;
	}

	/**Aftercare**/
	result = DBexecute("replace into ja_run_value_after_table select * from ja_run_value_before_table"
				" where inner_job_id = " ZBX_FS_UI64, ar.jobid);

	if ( ZBX_DB_OK > result) {
		ja_log("JATRAPPER200013", inner_jobnet_id, NULL, ar.jobid);
		ret = FAIL;
		return ret;
	}

	if ( SUCCEED != ja_set_value_after(ar.jobid, inner_jobnet_id, STD_OUT, ar.jobstdout))
	{
		ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid);
		ret = FAIL;
		return ret;

	}

	if ( SUCCEED != ja_set_value_after(ar.jobid, inner_jobnet_id, STD_ERR, ar.jobstderr))
	{
		ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid);
		ret = FAIL;
		return ret;

	}

	if ( SUCCEED != ja_set_value_after(ar.jobid, inner_jobnet_id, RET, ar.ret))
	{
		ja_log("JATRAPPER200011", inner_jobnet_id, NULL, ar.jobid);
		ret = FAIL;
		return ret;

	}

	if ( status == JA_JOBNET_STATUS_END && job_type != JA_JOB_TYPE_ABORT ) {
		if ( SUCCEED != ja_flow(ar.jobid, JA_FLOW_TYPE_NORMAL))
		{
			ret = FAIL;
			return ret;
		}
	}

	if  ( status == JA_JOBNET_STATUS_RUNERR || status == JA_JOBNET_STATUS_ENDERR ) {
		ja_joblog(JC_JOB_ERR_END, inner_jobnet_id, ar.jobid);
		ja_log("JATRAPPER200043", inner_jobnet_id, NULL, ar.jobid);
	}

	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
	return ret;
}

int register_db_table(char *jobnet_id)
{
	const char *__function_name = "register_db_table";
	char		*user_name = NULL;
	char		*jobnet_name = NULL;
	char		*memo = NULL;
	char		*jobnetid;
	int 		ret = SUCCEED;
	int		res;
	int		public_flag=0;
	zbx_uint64_t	next_id;
	zbx_uint64_t	update_date;
	DB_RESULT	result;
	DB_ROW		row;

	zabbix_log(LOG_LEVEL_DEBUG, "In %s() jobnetid: [%s]",__function_name, jobnet_id);
	
	jobnetid = DBdyn_escape_string(jobnet_id);
	result = DBselect("select update_date, created_date, public_flag, user_name, jobnet_name, memo" 
				" from ja_jobnet_control_table" 
				" where jobnet_id='%s'"
				" and valid_flag=1"
				, jobnetid);
	row = DBfetch(result);
	if ( row == NULL) {
		ja_log("JATRAPPER200014", 0, jobnet_id, jobnet_id);
		zbx_free(jobnet_id);
		DBfree_result(result);
		ret = FAIL;
		return ret;
	}

	DBfree_result(result);
	ZBX_STR2UINT64(update_date, row[0]);
	public_flag = atoi(row[2]);
	user_name = row[3];
	jobnet_name = row[4];
	memo = row[5];

	next_id = get_next_id(JA_RUN_ID_JOBNET_EX, 0, jobnet_id);

	if ( next_id == 0 )
	{
		ja_log("JATRAPPER200015", 0, jobnet_id);
		zbx_free(jobnet_id);
		ret = FAIL;
		return ret;
	}

	res = DBexecute("insert into ja_run_jobnet_table"
			" (inner_jobnet_id, inner_jobnet_main_id,"
			" update_date, run_type, main_flag, status, public_flag,"
			" jobnet_id, user_name, jobnet_name, memo)"
			" values ( "ZBX_FS_UI64", "ZBX_FS_UI64", "ZBX_FS_UI64","
			" 1, 0, 0, %d, '%s', '%s', '%s', '%s')"
			, next_id, next_id, update_date, public_flag, jobnetid, user_name, jobnet_name, memo);
	if ( ZBX_DB_OK > res) {
		ja_log("JATRAPPER200016", 0, jobnet_id, jobnet_id);
		zbx_free(jobnet_id);
		ret = FAIL;
		return ret;
	}

	zbx_free(jobnet_id);
	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
	return ret;
}

static int process_jatrap(zbx_sock_t * sock, char *s)
{
	int 		ret = SUCCEED;
	struct 		zbx_json_parse jp, jp_row;
	struct zbx_json 	json;
	char		value[MAX_STRING_LEN];
	char		*message = NULL;
	const char	*p;
	int		version;
	int		runflag = 0;
	static JOBARG_AGENT_RESULT	ar;
	static JOBARG_EXEC_REQUEST	er;
	zbx_uint64_t	jobid;

	zbx_rtrim(s, " \r\n");
	zabbix_log(LOG_LEVEL_DEBUG, "Jatrapper got [%s] len %zd", s, strlen(s));

	if ( SUCCEED == zbx_json_open(s, &jp))
	{
		if (SUCCEED == zbx_json_value_by_name(&jp, JA_PROTO_TAG_KIND, value, sizeof(value)))
		{
			/*****jobnetresult******/
			if (0 == strcmp(value, JA_PROTO_VALUE_JOBRESULT)) {

				init_agent_result(&ar);

				if (SUCCEED == zbx_json_value_by_name(&jp, JA_PROTO_TAG_VERSION, value, sizeof(value)))
				{
					version = atoi(value);
					if ( version != JA_PROTO_VALUE_VERSION_1) {
						ja_log("JATRAPPER200017", 0, NULL, JA_PROTO_VALUE_VERSION_1);
						message = zbx_dsprintf(message, "Received message error: [version] is not correct.");
						ret = FAIL;
						goto response;
					}
				} else {
					ja_log("JATRAPPER200018", 0, NULL);
					message = zbx_dsprintf(message, "Received message error: [version] not found");
					ret = FAIL;
					goto response;
				}
        	
				if (SUCCEED == zbx_json_value_by_name(&jp, JA_PROTO_TAG_SERVERID, value, sizeof(value)))
				{
					if (0 != strcmp(value, serverid)) {
						ja_log("JATRAPPER200019", 0, NULL, serverid, value);
						message = zbx_dsprintf(message, "Received message error: [serverid] is not correct.");
						ret = FAIL;
						goto response;
					}

				} else {
					ja_log("JATRAPPER200020", 0, NULL);
					message = zbx_dsprintf(message, "Received message error: [serverid] not found");
					ret = FAIL;
					goto response;

				}

				if (NULL == (p = zbx_json_pair_by_name(&jp, JA_PROTO_TAG_DATA))) {
					ja_log("JATRAPPER200021", 0, NULL);
					message = zbx_dsprintf(message, "Received message error: [data] not found");
					ret = FAIL;
					goto response;
				} else {
					if (FAIL == (ret = zbx_json_brackets_open(p, &jp_row))) {
						ja_log("JATRAPPER200022", 0, NULL);
						message = zbx_dsprintf(message, "Received message error: Cannot open [data] object");
						ret = FAIL;
						goto response;

					} else {
						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_HOST, value, sizeof(value)))
						{
							ar.host = strdup(value);

						} else {
							ja_log("JATRAPPER200023", 0, NULL);
							message = zbx_dsprintf(message, "Received message error: [host] not found");
							ret = FAIL;
							goto response;
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_JOBID, value, sizeof(value)))
						{
							ZBX_STR2UINT64(jobid, value);
							ar.jobid = jobid;
						} else {
							ja_log("JATRAPPER200024", 0, NULL);
							message = zbx_dsprintf(message, "Received message error: [jobid] not found");
							ret = FAIL;
							goto response;
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_METHOD, value, sizeof(value)))
						{
							ar.method = atoi(value);
							if ( ar.method < JA_AGENT_METHOD_NORMAL || ar.method > JA_AGENT_METHOD_KILL ) {
								ja_log("JATRAPPER200040", 0, NULL, ar.method);
								message = zbx_dsprintf(message, "Received message error: method range error [%d]", ar.method);
								ret = FAIL;
								goto response;
							}
						} else {
							ja_log("JATRAPPER200041", 0, NULL);
							message = zbx_dsprintf(message, "Received message error: [method] not found");
							ret = FAIL;
							goto response;
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_RESULT, value, sizeof(value)))
						{
							ar.result = atoi(value);
							if ( 0 != ar.result && 1 != ar.result ) {
								ja_log("JATRAPPER200025", 0, NULL, ar.result);
								message = zbx_dsprintf(message, "Received message error: result range error [%d]", ar.result);
								ret = FAIL;
								goto response;
							}
						} else {
							ja_log("JATRAPPER200026", 0, NULL);
							message = zbx_dsprintf(message, "Received message error: [result] not found");
							ret = FAIL;
							goto response;
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_MESSAGE, value, sizeof(value)))
						{
							ar.message = strdup(value);
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_JOBSTDOUT, value, sizeof(value)))
						{
							ar.jobstdout = strdup(value);
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_JOBSTDERR, value, sizeof(value)))
						{
							ar.jobstderr = strdup(value);
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_RET, value, sizeof(value)))
						{
							ar.ret = strdup(value);
						}

						if ( SUCCEED == (host_auth(sock, ar.host)))
						{
							DBbegin();
							if ( SUCCEED == (update_db_status(ar)))
							{
								DBcommit();
								message = zbx_dsprintf(message, "OK.");
							} else {
								DBrollback();
								message = zbx_dsprintf(message, "Update table error.");
								ret = FAIL;
 							}
						} else {
							message = zbx_dsprintf(message, "[host] is not authenticated");
							goto response;
						}
					}
				}
			/*****jobnetrun******/
			} else if (0 == strcmp(value, JA_PROTO_VALUE_JOBNETRUN)) {

				runflag = 1;
				init_exec_request(&er);

				if (SUCCEED == zbx_json_value_by_name(&jp, JA_PROTO_TAG_VERSION, value, sizeof(value)))
				{
					version = atoi(value);
					if ( version != JA_PROTO_VALUE_VERSION_1) {
						ja_log("JATRAPPER200027", 0, NULL, JA_PROTO_VALUE_VERSION_1);
						message = zbx_dsprintf(message, "Received message error: [version] is not correct.");
						ret = FAIL;
						goto response;
					}
				} else {
					ja_log("JATRAPPER200028", 0, NULL);
					message = zbx_dsprintf(message, "Received message error: [version] not found");
					ret = FAIL;
					goto response;
				}

				if (NULL == (p = zbx_json_pair_by_name(&jp, JA_PROTO_TAG_DATA))) {
					ja_log("JATRAPPER200029", 0, NULL);
					message = zbx_dsprintf(message, "Received message error: [data] not found");
					ret = FAIL;
					goto response;
				} else {
					if (FAIL == (ret = zbx_json_brackets_open(p, &jp_row))) {
						ja_log("JATRAPPER200030", 0, NULL);
						message = zbx_dsprintf(message, "Received message error: Cannot open [data] object");
						ret = FAIL;
						goto response;

					} else {
						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_HOST, value, sizeof(value)))
						{
							er.host = strdup(value);

						} else {
							ja_log("JATRAPPER200031", 0, NULL);
							message = zbx_dsprintf(message, "Received message error: [host] not found");
							ret = FAIL;
							goto response;
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_USERNAME, value, sizeof(value)))
						{
							er.username = strdup(value);

						} else {
							ja_log("JATRAPPER200032", 0, NULL);
							message = zbx_dsprintf(message, "Received message error: [username] not found");
							ret = FAIL;
							goto response;
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_PASSWORD, value, sizeof(value)))
						{
							er.password = strdup(value);

						} else {
							ja_log("JATRAPPER200033", 0, NULL);
							message = zbx_dsprintf(message, "Received message error: [password] not found");
							ret = FAIL;
							goto response;
						}

						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_JOBNETID, value, sizeof(value)))
						{
							er.jobnetid = strdup(value);

						} else {
							ja_log("JATRAPPER200034", 0, NULL);
							message = zbx_dsprintf(message, "Received message error: [jobnetid] not found");
							ret = FAIL;
							goto response;
						}

						if ( SUCCEED == (job_exec_auth(er)) )
						{
							DBbegin();
							if ( SUCCEED == (register_db_table(er.jobnetid)))
							{
								DBcommit();
								message = zbx_dsprintf(message, "OK.");
							} else {
								DBrollback();
								message = zbx_dsprintf(message, "Insert table error.");
								ret = FAIL;
 							}
							
						} else {
							message = zbx_dsprintf(message, "Authentication failure.");
							ret = FAIL;
							goto response;
						}
					}
				}

			} else {
				ja_log("JATRAPPER200035", 0, NULL);
				message = zbx_dsprintf(message, "Received message error: [kind] is not [jobresult] or [jobnetrun]");
				ret = FAIL;
				goto response;
			}

		} else {
			ja_log("JATRAPPER200036", 0, NULL);
			message = zbx_dsprintf(message, "Received message error: [kind] not found");
			ret = FAIL;
			goto response;
		}
	} else {
		ja_log("JATRAPPER200037", 0, NULL);
		message = zbx_dsprintf(message, "Cannot open received data");
		ret = FAIL;

	}

response:
	/*JSON Data*/
	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
	zbx_json_addstring(&json, JA_PROTO_TAG_KIND, runflag == 1 ? JA_PROTO_VALUE_JOBNETRUN_RES : JA_PROTO_VALUE_JOBRESULT_RES, ZBX_JSON_TYPE_STRING);
	zbx_json_adduint64(&json, JA_PROTO_TAG_VERSION, JA_PROTO_VALUE_VERSION_1); /*data version*/
	zbx_json_addstring(&json, JA_PROTO_TAG_SERVERID, serverid, ZBX_JSON_TYPE_STRING);
	zbx_json_addobject(&json, JA_PROTO_TAG_DATA);
	zbx_json_adduint64(&json, JA_PROTO_TAG_RESULT, FAIL == ret ? 1 : 0);
	zbx_json_addstring(&json, JA_PROTO_TAG_MESSAGE, message, ZBX_JSON_TYPE_STRING);
	zbx_json_close(&json);
	zbx_json_close(&json);

	zabbix_log(LOG_LEVEL_DEBUG, "JSON before sending [%s]", json.buffer);

	if (FAIL == (ret = zbx_tcp_send_to(sock, json.buffer, CONFIG_TIMEOUT)))
	{
		ja_log("JATRAPPER200038", 0, NULL, zbx_tcp_strerror());
		ret = NETWORK_ERROR;
	}

	return ret;
}

static void process_jatrapper(zbx_sock_t * sock)
{
	char *data;

    if (SUCCEED != zbx_tcp_recv_to(sock, &data, CONFIG_TIMEOUT))
        return;

    process_jatrap(sock, data);
}

void main_jatrapper_loop(zbx_sock_t * s)
{
    const char *__function_name = "main_jatrapper_loop";

    zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);

    zbx_setproctitle("%s [connecting to the database]",
                     get_process_type_string(process_type));

    DBconnect(ZBX_DB_CONNECT_NORMAL);

    for (;;) {
        zbx_setproctitle("%s [waiting for connection]",
                         ja_get_process_type_string(process_type));

        ja_update_selfmon_counter(ZBX_PROCESS_STATE_IDLE);

        if (SUCCEED == zbx_tcp_accept(s)) {
            ja_update_selfmon_counter(ZBX_PROCESS_STATE_BUSY);

            zbx_setproctitle("%s [processing data]",
                             get_process_type_string(process_type));

            process_jatrapper(s);

            zbx_tcp_unaccept(s);
        } else {
	    ja_log("JATRAPPER200039", 0, NULL);
	}
    }
}
